#include "TROOT.h"
#include "TFile.h"
#include "TList.h"
#include "TKey.h"
#include "TH1.h"
#include "TH2.h"
#include "TH3.h"
#include "TProfile.h"

#include <iostream>
#include <iomanip>
#include <string>

#include "CHistManager.hh"
#include "NGlobal.hh"

TH1* CHistManager::GetHist(std::string name)
{
  //  TObject* obj = gROOT->FindObject(name.c_str());
  TObject* obj = gROOT->FindObjectAny(name.c_str());

  if(obj && obj->InheritsFrom("TH1")) return (TH1*)obj;

  std::ostringstream os;
  os << FunctionName("GetHist") << name << " dosn't exist";
  NGlobal::Message(os.str(), kMID);

  return 0;
}

TH1* CHistManager::GetHist(int id)
{
  std::string name = GetName(id);

  if(!Check(name)) return 0;

  return GetHist(name);
}

TH1* CHistManager::GetHist()
{
  std::string name = GetName();  

  if(!Check(name)) return 0;

  return GetHist(name);
}

void CHistManager::GetNameXYZ(std::string title, std::string& namex, std::string& namey, std::string& namez)
{
  if(title.empty()) title = GetTitle();
  std::string namea, nameb, namec;

  namea = Split(title, ":");
  nameb = Split(title, ":");
  namec = Split(title, ":");

  if(namea.empty()){
    namex = "";
    namey = "";    
    namez = "";    
  }else if(nameb.empty()){
    namex = namea;
    namey = "";    
    namez = "";    
  }else if(namec.empty()){
    namex = nameb;
    namey = namea;    
    namez = "";    
  }else{
    namex = namec;
    namey = nameb;    
    namez = namea;     
  }
  
  std::ostringstream os;
  os << FunctionName("GetNameXYZ") << namex << " "  << namey << " " << namez;
  NGlobal::Message(os.str(), kDEBUG);
}

std::string CHistManager::GetNameX()
{
  std::string namex, namey, namez;
  GetNameXYZ("", namex, namey, namez);

  return namex;
}

std::string CHistManager::GetNameY()
{
  std::string namex, namey, namez;
  GetNameXYZ("", namex, namey, namez);

  return namey;
}

std::string CHistManager::GetNameZ()
{
  std::string namex, namey, namez;
  GetNameXYZ("", namex, namey, namez);

  return namez;
}

std::string CHistManager::Split(std::string& str, std::string keyword)
{
  if(str.empty()) return "";

  std::string::size_type index = str.find(keyword);
  if(index == std::string::npos){
    index = str.size();
  }
  std::string substr = str.substr(0, index);
  if(index+keyword.size() < str.size()){
    str = str.substr(index+keyword.size());
  }else{
    str = "";
  }

  return substr;
}

void CHistManager::Insert(std::string name, std::string title, int order, int id)
{
  IArrayManager::Insert(name, title, order, id);
  ApplyOption();  
}

void CHistManager::InsertAfterCurrent(std::string name, std::string title)
{
  int id = GetId();
  if(!Check(id)) return;

  Insert(name, title, id, -1);
}

void CHistManager::Insert(TFile* file, int offset)
{
  TIter nextkey(file->GetListOfKeys());
  TKey* key;
  TKey* oldkey = 0;

  bool init = true;
  CdequeIdNameTitle::iterator it;

  while ((key = (TKey*)nextkey())){
    //keep only the highest cycle number for each key
    if (oldkey && !strcmp(oldkey->GetName(), key->GetName())) continue;
    TObject *obj = key->ReadObj();
    if (obj->InheritsFrom("TH1")){
      TH1* hist = (TH1*)obj;
      Insert(hist->GetName(), hist->GetTitle(), offset, -1);
      if(!(offset < 0)){
	++offset;
      }
      if(init) it = GetCurrent();
      init = false;
    }
  }
  SetCurrent(it);
}

void CHistManager::Draw(std::string name)
{
  CdequeIdNameTitle::iterator it = FindByName(name);
  if(!Check(it)) return;

  SetCurrent(it);
  TH1* h = GetHist(name);
  if(!Check(h)) return;

  h->DrawCopy();
}

void CHistManager::Draw(int id)
{
  std::string name = GetName(id);
  if(!Check(name)) return ;

  Draw(name);
}

void CHistManager::Write()
{
  CdequeIdNameTitle::iterator it = Begin();
  while(it != End()){
    TH1* hist = GetHist(it->second);
    if(!Check(hist)) return;
    hist->Write();
    ++it;
  }    

  std::ostringstream os;
  os << FunctionName("Write") << "write hists";
  NGlobal::Message(os.str(), kLOW);
}

void CHistManager::SetDrawOption(std::string foption)
{
  fdrawOption = foption;
}

void CHistManager::SetDrawOption()
{
  SetDrawOption(fdrawOptionDef);
}

void CHistManager::ApplyOption(std::string name)
{
  TH1* hist = GetHist(name);
  if(!Check(hist)){
    std::ostringstream os;
    os << FunctionName("ApplyOption") << "not valid pointer";
    NGlobal::Message(os.str(), kMID);

    return;
  }

  hist->SetOption(fdrawOption.c_str());

  if(!hist->InheritsFrom("TH2") && !hist->InheritsFrom("TH3")){
    hist->SetFillColor(ffillColor);
  }
}

void CHistManager::ApplyOption(int id)
{
  CdequeIdNameTitle::iterator it = FindById(id);
  if(!Check(it)) return;
  
  ApplyOption(it->second);
}

void CHistManager::ApplyOption()
{
  CdequeIdNameTitle::iterator it = GetCurrent();
  if(!Check(it)) return;

  ApplyOption(it->second);
}

void CHistManager::ApplyOptionAll()
{
  CdequeIdNameTitle::iterator it = idnametitle.begin();  
  while(it != End()){
    ApplyOption(it->second);
    ++it;
  }
}

void CHistManager::SetFillColor(Color_t fcolor)
{
  ffillColor = fcolor;
}

void CHistManager::SetFillColor()
{
  SetFillColor(ffillColorDef);
}

std::string CHistManager::FunctionName(std::string functionname) const
{
  return "[CHistManager::" + functionname + "]$ ";
}

